package testsam;

import java.io.IOException;
import java.security.Principal;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.security.auth.Subject;
import javax.security.auth.callback.Callback;
import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.callback.UnsupportedCallbackException;
import javax.security.auth.message.AuthException;
import javax.security.auth.message.AuthStatus;
import javax.security.auth.message.MessageInfo;
import javax.security.auth.message.MessagePolicy;
import javax.security.auth.message.callback.CallerPrincipalCallback;
import javax.security.auth.message.callback.GroupPrincipalCallback;
import javax.security.auth.message.module.ServerAuthModule;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;


public class Sam implements ServerAuthModule {
	
	private static Logger LOGGER = Logger.getLogger(Sam.class.getName());
	
	private CallbackHandler handler;
	
	@SuppressWarnings("rawtypes")
	protected static final Class[] supportedMessageTypes = new Class[]{HttpServletRequest.class,HttpServletResponse.class};

	@Override
	public void initialize(MessagePolicy requestPolicy, MessagePolicy responsePolicy, CallbackHandler handler, @SuppressWarnings("rawtypes") Map options) throws AuthException {
		this.handler = handler;
	}

	@Override
	public AuthStatus validateRequest(MessageInfo info, Subject clientSubject, Subject serviceSubject) throws AuthException {
		HttpServletRequest req = (HttpServletRequest) info.getRequestMessage();
	
		String samUser = req.getParameter("user");
		if(samUser!=null){
			Principal p = null;
			String[] groups = null;
			
			if("Tom".equals(samUser)){
				p = new Principal() {
					@Override
					public String getName() {
						return "Tom";
					}
				};
				groups = new String[] {"user"};
			}else if("Jerry".equals(samUser)){
				p = new Principal() {
					@Override
					public String getName() {
						return "Jerry";
					}
				};
				groups = new String[] {"user", "sec-user"};
			}
			
			if(p!=null){
				clientSubject.getPrincipals().add(p);
				CallerPrincipalCallback callerCallback = new CallerPrincipalCallback(clientSubject, p);
				GroupPrincipalCallback groupCallback = new GroupPrincipalCallback(clientSubject, groups);
				
				try {
					handler.handle(new Callback[] { callerCallback, groupCallback });
				} catch (IOException e) {
					LOGGER.log(Level.SEVERE, "SAM Exception", e);
				} catch (UnsupportedCallbackException e) {
					LOGGER.log(Level.SEVERE, "SAM Exception", e);
				}
				
				LOGGER.info("SAM: principal = "+p.getName());
			}else{
				LOGGER.info("SAM: no principal");
			}
		}
		return AuthStatus.SUCCESS;
	}
	
	@Override
	public void cleanSubject(MessageInfo msgInfo, Subject subject) throws AuthException {
		if (subject != null) {
			subject.getPrincipals().clear();
		}
	}

	@Override
	public AuthStatus secureResponse(MessageInfo info, Subject serviceSubject) throws AuthException {
			return AuthStatus.SEND_SUCCESS;
	}

	@Override
	@SuppressWarnings("rawtypes")
	public Class[] getSupportedMessageTypes() {
		return supportedMessageTypes;
	}
}
